#include "common.h"

#define MAX_BUTTONS 100

typedef struct
{
  int exitcode;
  char *label;
} tbuttons;

int chosen;

/*************************************************************************/
void button_clicked(GtkWidget *Dummy, int number)
{
  chosen = number;
  gtk_main_quit();
}

/*************************************************************************/
int gmessage(const char *message, tbuttons *buttons,
             int nb_buttons, int _default, int print)
{
  GtkWidget *window;
  GtkWidget *vbox, *hbox, *scrolled_window;
  GtkText *text;
  int i;

  chosen = _default;
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "gmessage");

  gtk_signal_connect(GTK_OBJECT(window), "destroy",
                     (GtkSignalFunc)gtk_main_quit, NULL);
  gtk_signal_connect(GTK_OBJECT(window), "delete_event",
                     (GtkSignalFunc)gtk_main_quit, NULL);
    
  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(window), vbox);
  gtk_widget_show(vbox);

  PACK11(vbox, scrolled_window = gtk_scrolled_window_new(NULL, NULL) );
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
                                 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_widget_set_usize(GTK_WIDGET(scrolled_window), 150, 100);
    
  text = GTK_TEXT(gtk_text_new(NULL, NULL));
  gtk_text_insert(text, NULL, NULL, NULL, message, -1);

  gtk_container_add(GTK_CONTAINER(scrolled_window), GTK_WIDGET(text));
  gtk_widget_show(GTK_WIDGET(text));

  PACK00(vbox, hbox = gtk_hbox_new(FALSE, 0));

  for (i = 0; i < nb_buttons; i++)
  {
    GtkWidget *button;
    
    PACK11(hbox, button = gtk_button_new_with_label(buttons[i].label));
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
                       GTK_SIGNAL_FUNC(button_clicked), (gpointer) i);
    if (i == _default)
      gtk_widget_grab_focus(button);
  }

  gtk_widget_show(window);
  gtk_main();

  return chosen >= 0 ? buttons[chosen].exitcode : 1;
}

/*************************************************************************/
int main(int argc, char **argv)
{
  char *message = NULL;
  tbuttons buttons[MAX_BUTTONS];
  int nb_buttons = 0, print = 0;
  char *cdefault = NULL;
  int _default = -1;
  int i;

  initlocale("gmessage");

  for (argc--, argv++; argc; argc--, argv++)
  {
    if (streq(*argv, "-file"))
    {
      FILE *f;
      struct stat s;

      argc--; argv++;
      if (!argc)
        die(_("missing the filename after -file\n"));
      if ((f = fopen(*argv, "r")) == NULL)
        die(_("cannot open file %s\n"), *argv);
      if (fstat(fileno(f), &s))
        die(_("cannot stat file %s\n"), *argv);
      if ((message = malloc(s.st_size)) == NULL)
        die(_("malloc failed\n"));
      if (fread(message, 1, s.st_size, f) != s.st_size)
        die(_("error reading\n"));
      if (fclose(f))
        die(_("fclose failed\n"));
    } else if (streq(*argv, "-buttons"))
    {
      char *p, *q, *r;
      int i;

      argc--; argv++;
      if (!argc)
        die(_("missing the comma-separated list of label:exitcode\n"));
      for (i = 0, p = q = strdup(*argv); i < MAX_BUTTONS && q; p = q + 1, i++)
      {
        buttons[i].label = p;
        if ((q = strchr(p, ',')))
          *q = 0;

        if ((r = strchr(p, ':')))
        {
          *r = 0;
          buttons[i].exitcode = atoi(r + 1);
        } else buttons[i].exitcode = 100 + 1 + i;
      }
      nb_buttons = i;
    } else if (streq(*argv, "-default"))
    {
      argc--; argv++;
      if (!argc)
        die(_("missing the label of the default button\n"));
      cdefault = *argv;
    } else if (streq(*argv, "-print"))
    {
      print = 1;
    } else if (streq(*argv, "-center"))
    {
      die(_("center not implemented\n"));
    } else if (streq(*argv, "-nearmouse"))
    {
      die(_("nearmouse not implemented\n"));
    } else if (streq(*argv, "-timeout"))
    {
      argc--; argv++;
      if (!argc)
        die(_("missing the number of seconds before timeout\n"));
      die(_("timeout not implemented\n"));
    } else break;
  }

  if (((message != 0) ^ argc) == 0)
     die(_("usage: gmessage [-options] [message ...] Type \'man gmessage\' for more help.\n"));
/* die(_(" \n")); */
/* die(_("where options include: \n")); */
/* die(_("    -file filename              file to read message from, \"-\" for stdin \n")); */
/* die(_("    -buttons string             comma-separated list of label:exitcode \n")); */
/* die(_("    -default button             button to activate if Return is pressed \n")); */
/* die(_("    -print                      print the button label when selected \n")); */
/* die(_("    -center                     pop up at center of screen \n")); */
/* die(_("    -nearmouse                  pop up near the mouse cursor \n")); */
/* die(_("    -timeout secs               exit with status 0 after \"secs\" seconds \n")); */
/* */
  if (argc)
  {
    message = g_strdup("");
    for (; argc; argc--, argv++)
    {
      gchar *p = g_strconcat(message, *argv, " ", NULL);
      g_free(message);
      message = p;
    }
  }

  if (nb_buttons == 0)
  {
    nb_buttons = 1;
    buttons[0].label = _("Ok");
    buttons[0].exitcode = 0;
  }

  if (cdefault)
    for (i = 0; i < nb_buttons; i++)
      if (streq(cdefault, buttons[i].label))
      { _default = i; break; }

  gtk_init(NULL, NULL);
  exit(gmessage(message, buttons, nb_buttons, _default, print));
}
